/*
   Little picture test program.

   I think some compression for the image could be usefull...

   Got the picture from http://gfxzone.planet-d.net, its made by AE ...
   and converted with gfx2gc. Converter can be downloaded at www.console-dev.de/ogc
*/

// Include OpenGC functions
#include <mygcn.h>

#include "now3d.raw.c"  // now3d_Bitmap[96][288]
#include "cd.raw.c"
#include "london.raw.c"


#include "sinus.h"



// duke released the assembler code of this on his page.
// so, here is the c/c++ version
#define REG_HOTRESET             (0xCC003024)
#define REG_HOTRESET_PTR         ((volatile u32*)REG_HOTRESET)
#define M_HOTRESET               *REG_HOTRESET_PTR = 0x00000000;


//#define DEBUG

#define RAND_A 9301
#define RAND_C 49297
#define RAND_M 233280

#define DOT_PRODUCT(ux,uy,vx,vy) ((ux)*(vx) + (uy)*(vy))

#define BOXES 20

typedef struct
{
  float vx;
  float vy;
}
VECTOR2D;


typedef struct
{
  float radius;
  int mass;
  VECTOR2D velocity;
  VECTOR2D position;
  int r, g, b;
}
object;

object boxes[BOXES];
object * temp;

/* Perfect, elastic collisions, not real world, but great for demos :)
   Using float, so no increase/decrease for overall total velocity in the system
   like PSX suffers from due to no FPU and having to use float emulation using int's
*/
static const float ecof = 1.0f;

/* Constants */
u32 seed = 0x12345678;
static const float VELOCITY = 6.0f;
u32 RADIUS = 5;
u32 MASS = 4;
u32 time = 0;
int finished = 0;
u32 g_photo_move = 0;
u32 g_wait_cycle = 0;
u32 g_collision_count = 0;

char text[] = "Yo, put this little GC demo together from some PS2 code I wrote a while back.  My logo, then photo of London at night, followed by Newtons conservation of momentum.  Greetings go out to: Dreamtime, Oobles, MikeT, GothicF and everyone else who knows me!";


void doImageEffect(void);
void doImage2Effect(void);
void doImage3Effect(void);

/* Functions */
int initialise_physics1(void);
int initialise_physics2(object * objs, int total_objects);
int initialise_physics(object * objs, int total_objects);

void physics_demo(void);
void check_edges(void);
int check_collisions(void);

int resolve_collision(object * box1, object * box2, float nabx, float naby, float length);
int initialise_object(object * obj);

int get_max_y(void);

void draw_ball(int x_pos, int y_pos);


u32 *g_pFb=(u32*)0xC0500000; // External framebuffer pointer

PAD g_Pad;

#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480

int get_max_x(void)
{
	return SCREEN_WIDTH;
}

int get_max_y(void)
{
	return SCREEN_HEIGHT;
}

int main()
{
	int init = 0;

	g_collision_count = 0;

    // Initialize video interface
    VIDEO_Init(VIDEO_640X480_NTSC_YUV16);

    // Set framebuffer1 address
    VIDEO_SetFrameBuffer(VIDEO_FRAMEBUFFER_BOTH, (u32)g_pFb);

    // init debugtext system
    DEBUG_InitText(g_pFb, COLOR_WHITE);

  init = initialise_physics(boxes, BOXES);
  

    while(1)
    {
      // clear external framebuffer
      RENDER_ClearFrameBuffer(g_pFb, COLOR_NAVY);

      // Read pad state
      PAD_ReadState(&g_Pad, PAD_CHANNEL_0);

      // start + A pressed?
      if(g_Pad.Digital.Start)
      {
         break;
      }

      // print some text
      DEBUG_Print(50, 50, "Conservation of momentum");
      DEBUG_Print(50, 70, "Quick port from PS2, by now3d.");
	  DEBUG_Print(50, 90, "Extra night photo of London by:");
	   DEBUG_Print(50, 110, "http://skyscrapernews.com/");
      DEBUG_Print(50, 150, "Press START to begin");

	   DEBUG_Print(50, 250, "Press START and A to end");

      // Wait for vertical retrace
      VIDEO_WaitVSync();
    }



	g_photo_move = 0;
	g_wait_cycle = 0;

    RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);

    while(1)
    {
		 RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);
      // Read pad state
      PAD_ReadState(&g_Pad, PAD_CHANNEL_0);

      // start + A pressed?
      if(g_Pad.Digital.B)
      {
         // reset
         break;
      }

      doImage2Effect();
	  if(g_photo_move < 100) g_photo_move ++;
	  else g_wait_cycle ++;

      // Wait for vertical retrace
      VIDEO_WaitVSync();

	  if (g_wait_cycle > 50 )
	  {
		  break;
	  }
    }
  


  	g_photo_move = 470;
	g_wait_cycle = 0;

    RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);

    while(1)
    {
		 RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);

      // Read pad state
      PAD_ReadState(&g_Pad, PAD_CHANNEL_0);

      // start + A pressed?
      if(g_Pad.Digital.B)
      {
         // reset
         break;
      }

      doImage3Effect();
	  if(g_photo_move >100) g_photo_move -= 3;
	  else g_wait_cycle ++;


      // Wait for vertical retrace
      VIDEO_WaitVSync();

	  if (g_wait_cycle > 50 )
	  {
		  break;
	  }
    }


   RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);

    while(1)
    {
		RENDER_ClearFrameBuffer(g_pFb, COLOR_BLACK);

      // Read pad state
      PAD_ReadState(&g_Pad, PAD_CHANNEL_0);

      // start + A pressed?
      if(g_Pad.Digital.Start && g_Pad.Digital.A)
      {
         // reset
         M_HOTRESET;
      }

	physics_demo();

	time++;


	DEBUG_Print(7, 340, text);

      // Wait for vertical retrace
      VIDEO_WaitVSync();
    }

    return 0;
}

inline void putpixel(int x, int y, u32 color)
{
   g_pFb[320*y+x]=color;
}


//16x16
void draw_ball(int x_pos, int y_pos)
{
   int x, y;

   int width=(CD_WIDTH>>1);
   int height=(CD_HEIGHT);

   for(y=0; y<height; y++)
   {
      for(x=0; x<width; x++)
      {
         putpixel(x + x_pos, y + y_pos, cd_Bitmap[y][x]);
      }
   }
}


void doImage2Effect(void)
{
   int x = 0;
   int y = 0;

   u32 colour = 0;


   int width=(NOW3D_WIDTH>>1);
   int height=(NOW3D_HEIGHT);


   for(y=0; y < height; y++)
   {
      for(x=0; x < width; x++)
      {
		 // colour = fade_colour(now3d_Bitmap[y][x]);
		 //temp_y = y *g_photo_move
		 colour = now3d_Bitmap[y][x];
         putpixel(x+50, y + g_photo_move, colour);
      }
   }
}

void doImage3Effect(void)
{
   int x = 0;
   int y = 0;

   u32 colour = 0;


   int width=(LONDON_WIDTH>>1);
   int height=(LONDON_HEIGHT);


   for(y=0; y < height; y++)
   {
      for(x=0; x < width; x++)
      {
		 // colour = fade_colour(now3d_Bitmap[y][x]);
		 //temp_y = y *g_photo_move
		 colour = london_Bitmap[y][x];
         putpixel(x + g_photo_move, y + 50, colour);
      }
   }
}





float crap_rand(void)
{
  seed = (seed * RAND_A + RAND_C) % RAND_M;
  return ((float) seed / (float) RAND_M);
}


int resolve_collision(object * box1, object * box2, float nabx, float naby, float length)
{
	float ma, mb;
	float tabx, taby;
	float vait, vain, vbit, vbin;
	float vafn, vbfn;
	float vaft, vbft;
	float xfa, yfa, xfb, yfb;

		/* Normalise by the distance between the two boxes */
		nabx /= length;
	   	naby /= length;

		/* tangential vector is perpendicular to normal, simply rotate vector 90 */
		tabx = -naby;
		taby = nabx;

		vait = DOT_PRODUCT(box1->velocity.vx, box1->velocity.vy, tabx, taby);

		vain = DOT_PRODUCT(box1->velocity.vx, box1->velocity.vy, nabx, naby);

		vbit = DOT_PRODUCT(box2->velocity.vx, box2->velocity.vy, tabx, taby);

		vbin = DOT_PRODUCT(box2->velocity.vx, box2->velocity.vy, nabx, naby);

		ma = mb = MASS;

		vafn = (mb * vbin * (ecof + 1) + vain * (ma - ecof * mb)) / (ma + mb);
		vbfn = (ma * vain * (ecof + 1) - vbin * (ma - ecof * mb)) / (ma + mb);

		/* tangential components is the same as before */
		vaft = vait;
		vbft = vbit;

		/* Change n,t from co-ords back to x,y co-ords */

		xfa = vafn * nabx + vaft * tabx;
		yfa = vafn * naby + vaft * taby;

		xfb = vbfn * nabx + vbft * tabx;
		yfb = vbfn * naby + vbft * taby;

		box1->velocity.vx = xfa;
		box1->velocity.vy = yfa;

		box2->velocity.vx = xfb;
		box2->velocity.vy = yfb;

		/* Update their positions to avoid another collision */
		box1->position.vx += box1->velocity.vx;
		box1->position.vy += box1->velocity.vy;

		box2->position.vx += box2->velocity.vx;
		box2->position.vy += box2->velocity.vy;

		g_collision_count ++;
	return 1;
}




/* Using conservation of momentum equasions calc:
  va2 = (e+1)*mb*vb1+va1(ma - e*mb)/(ma+mb)
  vb2 = (e+1)*ma*va1+vb1(ma - e*mb)/(ma+mb)
*/
int check_collisions(void)
{
  int ball_a = 0;
  int ball_b = 0;
  float nabx, naby, length;
  float ma, mb;
  float tabx, taby;
  float vait, vain, vbit, vbin;
  float vafn, vbfn;
  float vaft, vbft;
  float xfa, yfa, xfb, yfb;

  for (ball_a = 0; ball_a < BOXES; ball_a++)
   {
     for (ball_b = ball_a + 1; ball_b < BOXES; ball_b++)
      {
	if (ball_a == ball_b)
	  continue;


	/* Calc the normal vector between the two compared boxes. length shows how close! */
	nabx = (boxes[ball_b].position.vx - boxes[ball_a].position.vx);
	naby = (boxes[ball_b].position.vy - boxes[ball_a].position.vy);
	length = sqrtf(nabx * nabx + naby * naby);

	/* Check for collision by using RADIUS of the disc */
	if (length <= 2.0f * RADIUS * 1.4f)
	 {

	   /* Normalise by the distance between the two boxes */
	   nabx /= length;
	   naby /= length;

	   /* tangential vector is perpendicular to normal, simply rotate vector 90 */
	   tabx = -naby;
	   taby = nabx;

	   vait = DOT_PRODUCT(boxes[ball_a].velocity.vx, boxes[ball_a].velocity.vy, tabx, taby);

	   vain = DOT_PRODUCT(boxes[ball_a].velocity.vx, boxes[ball_a].velocity.vy, nabx, naby);

	   vbit = DOT_PRODUCT(boxes[ball_b].velocity.vx, boxes[ball_b].velocity.vy, tabx, taby);

	   vbin = DOT_PRODUCT(boxes[ball_b].velocity.vx, boxes[ball_b].velocity.vy, nabx, naby);

	   ma = mb = MASS;

	   vafn = (mb * vbin * (ecof + 1) + vain * (ma - ecof * mb)) / (ma + mb);
	   vbfn = (ma * vain * (ecof + 1) - vbin * (ma - ecof * mb)) / (ma + mb);

	   /* tangential components is the same as before */
	   vaft = vait;
	   vbft = vbit;

	   /* Change n,t from co-ords back to x,y co-ords */

	   xfa = vafn * nabx + vaft * tabx;
	   yfa = vafn * naby + vaft * taby;

	   xfb = vbfn * nabx + vbft * tabx;
	   yfb = vbfn * naby + vbft * taby;

	   boxes[ball_a].velocity.vx = xfa;
	   boxes[ball_a].velocity.vy = yfa;

	   boxes[ball_b].velocity.vx = xfb;
	   boxes[ball_b].velocity.vy = yfb;

	   /* Update their positions to avoid another collision */
	   boxes[ball_a].position.vx += boxes[ball_a].velocity.vx;
	   boxes[ball_a].position.vy += boxes[ball_a].velocity.vy;

	   boxes[ball_b].position.vx += boxes[ball_b].velocity.vx;
	   boxes[ball_b].position.vy += boxes[ball_b].velocity.vy;

	   g_collision_count ++;
	 }
      }
   }
   return 0;
}







void physics_demo(void)
{
  u32 loop = 0;
  u32 temp_x = 0;
  u32 temp_y = 0;

  for (loop = 0; loop < BOXES; loop++)
   {

	temp_x = boxes[loop].position.vx - boxes[loop].radius;
	temp_y =  boxes[loop].position.vy - boxes[loop].radius;

	if (loop == 0)
	{
		DEBUG_Print(20,20, "Collisions: %d", g_collision_count);

		//DEBUG_Print(20,20, "x: %d, y: %d", temp_x, temp_y);
	}
	//DEBUG_Print(temp_x+20, temp_y+20, "B");
    
	draw_ball(temp_x, temp_y);
   }


  /* Check if boxes are over the edge of the screen */
  check_edges();

  /* Check for collisions with other boxes, then calc the new velocity */
  check_collisions();
  //check_collisions2();


/* When debugging slow down the updates based on velocity */
#ifdef DEBUG
  if (time % 10 == 0)
   {
#endif
     // update positions
     for (loop = 0; loop < BOXES; loop++)
      {
	boxes[loop].position.vx += boxes[loop].velocity.vx;
	boxes[loop].position.vy += boxes[loop].velocity.vy;
      }
#ifdef DEBUG
   }
#endif

}


/* Check of boxes are off the screen on any of the 4 sides, and invert 
   their velocity in that direction. Include extra velocity update to avoid
   boxes being trapped at edge of the screen.
*/
void check_edges(void)
{
  u32 loop;

  for (loop = 0; loop < BOXES; loop++)
   {
     if ((boxes[loop].position.vx + boxes[loop].velocity.vx + RADIUS) > get_max_x() - 20)
     {
       boxes[loop].velocity.vx *= -1;
       boxes[loop].position.vx += boxes[loop].velocity.vx;
     }

     if ((boxes[loop].position.vy + boxes[loop].velocity.vy + RADIUS) > get_max_y() - 20)
       {
        boxes[loop].velocity.vy *= -1;
	boxes[loop].position.vy += boxes[loop].velocity.vy;
       }


     // use + for velocity to evaluate its true sign, +- evaluates to -   but  -- evaluates to + !!
     if ((boxes[loop].position.vx + boxes[loop].velocity.vx - RADIUS) < 20)
     {
       boxes[loop].velocity.vx *= -1;
       boxes[loop].position.vx += boxes[loop].velocity.vx;
     }

     if ((boxes[loop].position.vy + boxes[loop].velocity.vy - RADIUS) < 20)
     {
       boxes[loop].velocity.vy *= -1;
       boxes[loop].position.vy += boxes[loop].velocity.vy;
     }
   }

}



/* Initialise each box in turn, return 0 on sucessful completion
   of all initialisations. */
int initialise_physics(object * objs, int total_objects)
{
	int loop = 0;
	int value = 0;
	int ret = 0;

	for (loop = 0; loop < total_objects; loop++)
	{
		value = initialise_object(&objs[loop]);
		ret |= value;
	}
	return ret;
}



int initialise_object(object * obj)
{

	obj->radius = RADIUS;
	obj->mass = MASS;
	obj->velocity.vx = (crap_rand() * VELOCITY) - VELOCITY / 2;
	obj->velocity.vy = (crap_rand() * VELOCITY) - VELOCITY / 2;

	/* Ensure boxes start in main area of screen */
	obj->position.vx = crap_rand() * (get_max_x() - 40) + 20;
	obj->position.vy = crap_rand() * (get_max_y() - 40) + 20;

	return 0;
}


