/*
***********************************
Sprite example, By DesktopMan
(DesktopMan[at]fritidsproblemer.no)
**********************************

Thanks to:	Costis, for inspiration, etc etc
			Peter from console-dev.de, for Pad source
			Titanik for init and example code
			#gcdev for not beeing lame ALL the time
			Torlus for gcc build
			Probably someone else, you know who you are

Use of any parts of this source is hereby granted.
A thank you would be nice though :)
www.gcdev.com

ps:

Lib used is now OpenGC, http://www.console-dev.de/ogc/

How to add different gfx:

Replace bitmap.bmp with the gfx in question, with each frame under eachother.
Define NUM_FRAMES as the correct number of frames. Run gfx.bat. Compile, you are ready to go.
If there is only one frame, you can remove the animation-code.

*/

// Change these to where you keep your libs

#include "..\..\lib\types.h"
#include "..\..\lib\debug.h"
#include "..\..\lib\video.h"
#include "..\..\lib\memory.h"
#include "..\..\lib\pad.h"
#include "..\..\lib\color.h"

#include ".\gfx\sprite.raw.c" // sprite gfx

#define COLOR_TRANSPARENT	COLOR_WHITE // White is transparent (see sprite.bmp and sprite.raw.c)
#define NUM_FRAMES			0x4 // Four pictures in the animation

// Hot-reset, mapped to Start. Very handy. Thanks to http://duke.napalm-x.com/ and Peter for this one
#define REG_HOTRESET             (0xCC003024)
#define REG_HOTRESET_PTR         ((volatile u32*)REG_HOTRESET)
#define M_HOTRESET               *REG_HOTRESET_PTR = 0x00000000;

// Framebuffer
u32* fb = (u32*)0xC0500000;

PAD pad;

// Sprite struct
struct ssprite{
	int x;
	int y;
	int width;
	int height;
	int frame; // Current frame-number
	int frames; // Total frame-number
}sprite;

// Function declarations
void GetInput();
void ClearScreen(int color);
void BlitScreen();

int frames; // vblank's ellapsed

int main()
{
    // Init video in PAL mode, change this to your video-mode.
    VIDEO_Init(VIDEO_640X480_PAL60_YUV16);
    VIDEO_SetFrameBuffer(VIDEO_FRAMEBUFFER_BOTH,(u32)fb);

	// Init sprite-data

	sprite.height=SPRITE_HEIGHT;
	sprite.width=SPRITE_WIDTH;
	sprite.frames=NUM_FRAMES;
	sprite.frame=0;

	sprite.x=50;
	sprite.y=50;

    while(1) // Main game loop
    {   
		VIDEO_WaitVSync(); // Wait for vblank

		frames++;

		PAD_ReadState(&pad, PAD_CHANNEL_0); // Read pad

		ClearScreen(COLOR_BLACK); // Paint the screen black
	
		BlitScreen(); // Draw paddles and ball
		GetInput(); // duh
		
		if(frames%8==0){ // Switch picture every 8th frame
		sprite.frame++;

		if(sprite.frame==sprite.frames)
			sprite.frame=0;
		}
		if(pad.Digital.Start) // Hot-reset if start is pressed
			M_HOTRESET;
	}    
    return 0;
}

void GetInput(){ // Controller input, add boundaries if you want. (if sprite goes out of screen, it can trash memory)

	if(pad.Analog.X>20)
		sprite.x++;
	if(pad.Analog.X<-20)
		sprite.x--;
	if(pad.Analog.Y>20)
		sprite.y++;
	if(pad.Analog.Y<-20)
		sprite.y--;

}

void ClearScreen(int color){ // Clear the screen in a certain color
	int x,y;

	for(y=0;y<480;y++){
		for(x=0;x<320;x++){
			fb[x+320*y]=color;
		}
	}
}

void BlitScreen(){ // Draw everything
	int x,y,bitmappos;
	for(y=sprite.y;y<sprite.y+(sprite.height/sprite.frames);y++){
		for(x=sprite.x;x<sprite.x+(sprite.width/2);x++){
			bitmappos=x-sprite.x+((y-sprite.y)*sprite.width)/2+(sprite.width/2)*(sprite.height/sprite.frames)*sprite.frame; // calculate offset in the sprite-data (messy, I know)
			if(sprite_Bitmap[bitmappos]!=COLOR_TRANSPARENT) // Plot only if the color is not transparent, remove this to plot everything
				fb[x+320*y]=sprite_Bitmap[bitmappos]; 
		}
	}
}
